from flask import request
from .restful_exceptions import BadRequest

    
def get_args_from_request():
    if request.is_json:
        request_args = request.get_json()
    else:
        request_args = request.args.to_dict()
    return request_args


class Parser:
    def __init__(self):
        self.args = []
    
    def add_argument(self, name, dest=None, required=False, type=None, choices=None, help=None, default=None):
        self.args.append({
            'name': name,
            'dest': dest or name,
            'required': required,
            'type': type,
            'choices': choices,
            'help': help,
            'default': default,
        })

    def parse_args(self, request_args, strict):
        args = {}
        if strict:
            extra_args = set(request_args.keys()) - set(arg['name'] for arg in self.args)
            if extra_args:
                raise BadRequest(f"Got unexpected arguments: {', '.join(extra_args)}")
    
        for arg in self.args:
            value = request_args.get(arg['name'], arg.get('default', None))
            if value is None and arg['required']:
                raise BadRequest(f"{arg['name']} is required.")
        
            if value is not None:
                if arg['type']:
                    try:
                        value = arg['type'](value)
                    except Exception:
                        message = f"{arg['name']} should be {arg['type'].__name__}."
                        raise BadRequest(message)
                if arg.get('choices') and value not in arg['choices']:
                    raise BadRequest(f"{arg['name']} must be one of {arg['choices']}.")
            args[arg['dest']] = value
        return args


class RequestParser(Parser):
    def parse_args(self, strict=False):
        request_args = get_args_from_request()
        return super().parse_args(request_args, strict)
    



from functools import wraps
from flask import jsonify
from .restful_exceptions import BadRequest, CustomException


def decorator_method(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            result = func(*args, **kwargs)
        except Exception as e:
            if isinstance(e, BadRequest):
                return jsonify({"message": str(e)}), 400
            elif isinstance(e, CustomException):
                return jsonify({"message": e.message}), e.status_code
            else:
                raise e
        
        if not isinstance(result, tuple) or len(result) != 2 or not isinstance(result[1], int):
            return jsonify(result), 200
        return result
    
    return wrapper



from flask import request, jsonify
from .decorator_method import decorator_method


class Api:
    """
    A class representing a RESTful API.
    """

    def __init__(self, app):
        """
        Constructor for the Api class.

        :param app: The Flask application to which the API will be added.
        """
        self.app = app
        self.resources = {}

    def add_resource(self, resource_class, *routes):
        """
        Adds a resource to the API.

        :param resource_class: The class of the resource to add.
        :param routes: One or more URL routes to associate with the resource.
        """
        resource = resource_class()
        endpoint = resource.class_name.lower()

        for i, route in enumerate(routes):
            self.resources[route] = resource
            self.app.add_url_rule(
                route,
                view_func=resource.as_view(endpoint+str(i)),
                methods=resource.methods
            )


class Resource:
    """
    A base class for RESTful resources.
    """
    methods = {"GET", "POST", "PUT", "DELETE"}

    @property
    def class_name(self):
        return str(self.__class__.__name__)
    
    def dispatch_request(self, *args, **kwargs):
        """
        Dispatches a request to the appropriate method on the resource.

        :param args: Positional arguments to pass to the method.
        :param kwargs: Keyword arguments to pass to the method.
        """
        request_method = request.method
        meth = getattr(self, request_method.lower(), None)
        if meth is None and request_method == 'HEAD':
            meth = getattr(self, 'get', None)
        if meth is None:
            return jsonify({"message": f'Unimplemented method {request_method}'}), 405

        meth = decorator_method(meth)
        return meth(*args, **kwargs)

    def as_view(self, endpoint):
        """
        Returns a Flask view function for the resource.

        :param endpoint: The endpoint name for the view function.
        """
        def view(*args, **kwargs):
            return self.dispatch_request(*args, **kwargs)

        # Add the methods to the view function
        for method in self.methods:
            view.__dict__[method.lower()] = True
        view.__name__ = endpoint  # Flask can generate URLs for the endpoint based on its name.
        return view



class BadRequest(Exception):
    pass


class CustomException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message


def abort(status_code=400, message="Reason not given"):
    raise CustomException(status_code, message)



from setuptools import (
    setup,
    find_packages
)


with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='simple_flask_restful',
    version='0.2.0',
    author='William Wang',
    author_email='williamwangatsydney@gmail.com',
    description='wrap up flask for restful interface',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/AutomationLover/simple_flask_restful',
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=requirements
)



"""
simple_flask_restful


This library is a lightweight Python package that provides a simpler interface to create RESTful APIs using Flask.
"""

from .simple_restful import Api, Resource
from .restful_exceptions import abort
from . import reqparse
from flask import Flask


__version__ = '0.2.0'
__all__ = ['Api', 'Resource', 'abort', 'reqparse', 'Flask']
